@@ -13,19 +13,16 @@
 # "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 # either express or implied.
 
+from __future__ import print_function
 import argparse
-import collections
 import json
 import filecmp
 import traceback
 import glob
-import urllib
-import urllib2
 import zipfile
-import threading
 import platform
 
-from repo import *
+from .repo import *
 
 def get_host_os():
     host_os = platform.system()
@@ -52,7 +49,7 @@ def cd_aos_root():
     while os.path.isdir('./include/aos') == False:
         os.chdir('../')
         if sys_root.match(os.getcwd()):
-            print "error: not in AliOS-Things source code directory"
+            print("error: not in AliOS-Things source code directory")
             return 'fail'
     return 'success'
 
@@ -74,85 +71,127 @@ def which(program):
                 return exe_file
     return None
 
+def download_url(url, file_name=None, retry=1, printlog=True):
+    try:
+        from urllib.request import urlopen
+    except:
+        from urllib2 import urlopen
+
+    if file_name == None:
+        file_name = url.split('/')[-1]
+    succeed = False
+    while retry > 0:
+        u = urlopen(url)
+        f = open(file_name, 'wb')
+        meta = u.info()
+        file_size = int(meta.getheaders("Content-Length")[0])
+
+        file_size_dl = 0
+        block_sz = 8192
+        while True:
+            buffer = u.read(block_sz)
+            if not buffer:
+                break
+
+            file_size_dl += len(buffer)
+            f.write(buffer)
+            if printlog:
+                status = r"Downloading: {0}({1} bytes)  [{2:3.2f}%]".format(file_name, file_size, file_size_dl * 100. / file_size)
+                status = status + chr(8)*(len(status)+1)
+                print(status, end='')
+        f.close()
+        if file_size == file_size_dl:
+            succeed = True
+            break
+        retry -= 1
+    if printlog: print('')
+    return succeed
+
 def __run_setup():
     host_os = get_host_os()
-    print "installing dependent packages"
-    if host_os == 'Linux64':
+    sys.stdout.flush()
+    packages = []
+    if host_os.startswith('Linux'):
+        fdevnull = open('/dev/null', 'w')
+        import getpass
+        sudo = 'sudo '
+        if getpass.getuser() == 'root':
+            sudo = ''
         if which('apt-get') != None:
-            popen('sudo apt-get -q update', shell=True)
-            install_cmd = 'sudo apt-get -yq install '
-            packages  = 'git wget make flex bison gperf unzip python-pip'
-            packages += ' gcc-multilib libssl-dev libssl-dev:i386'
-            packages += ' libncurses-dev libncurses-dev:i386'
-            packages += ' libreadline-dev libreadline-dev:i386'
+            pre_install_cmd = []
+            if host_os.endswith('64') and 'i386' not in subprocess.check_output(['dpkg', '--print-foreign-architectures']):
+               pre_install_cmd += [sudo+'dpkg --add-architecture i386']
+            pre_install_cmd += [sudo+'apt-get -qq update']
+            install_cmd = sudo+'apt-get -yqq install '
+            check_installed_cmd = 'dpkg -s '
+            packages  = ['git', 'wget', 'make', 'flex', 'bison', 'gperf', 'unzip']
+            packages += ['libssl-dev', 'libncurses5-dev', 'libreadline-dev']
+            if host_os.endswith('64'):
+                packages += ['gcc-multilib', 'g++-multilib', 'libssl-dev:i386', 'libncurses5-dev:i386', 'libreadline-dev:i386']
         elif which('yum') != None:
-            install_cmd = 'sudo yum -yq install '
-            packages  = 'git wget make flex bison gperf unzip python-pip'
-            packages += ' gcc gcc-c++ glibc-devel glibc-devel.i686 libgcc libgcc.i686'
-            packages += ' libstdc++-devel libstdc++-devel.i686'
-            packages += ' openssl-devel openssl-devel.i686'
-            packages += ' ncurses-devel ncurses-devel.i686'
-            packages += ' readline-devel readline-devel.i686'
+            pre_install_cmd = [sudo+'yum -yq update']
+            install_cmd = sudo+'yum -yq install '
+            check_installed_cmd = 'yum list installed '
+            packages  = ['git', 'wget', 'make', 'flex', 'bison', 'gperf', 'unzip']
+            packages += ['gcc', 'gcc-c++', 'glibc-devel', 'libgcc, ''libstdc++-devel']
+            packages += ['openssl-devel', 'ncurses-devel', 'readline-devel']
+            if host_os.endswith('64'):
+                packages += ['glibc-devel.i686', 'libgcc.i686', 'libstdc++-devel.i686']
+                packages += ['openssl-devel.i686', 'ncurses-devel.i686', 'readline-devel.i686']
         else:
-            print "error: unsupported linux distribution"
-        popen(install_cmd + packages, shell=True)
-    elif host_os == 'Linux32':
-        if which('apt-get') != None:
-            popen('sudo apt-get update', shell=True)
-            install_cmd = 'sudo apt-get -yq install '
-            packages  = 'git wget axel make flex bison gperf unzip python-pip'
-            packages += ' libssl-dev libncurses-dev libreadline-dev'
-        elif which('yum') != None:
-            install_cmd = 'sudo yum -yq install '
-            packages  = 'git wget make flex bison gperf unzip python-pip'
-            packages += ' gcc gcc-c++ glibc-devel libgcc libstdc++-devel'
-            packages += ' openssl-devel ncurses-devel readline-devel'
-        else:
-            print "error: unkonw linux version"
-        popen(install_cmd + packages, shell=True)
-    elif host_os == 'OSX':
-        if which('pip') == None:
-            popen('sudo easy_install pip', shell=True)
+            print('Warning: not yet supported linux distribution')
+        for package in list(packages):
+            if subprocess.call((check_installed_cmd+package).split(), stdout=fdevnull, stderr=fdevnull) != 0:
+                continue
+            packages.remove(package)
 
     downloads = {}
     if os.path.isdir('./build/compiler') == False:
         toolchains = {
-            'Win32': 'https://code.aliyun.com/vivid8710/aos-toolchain-windows/repository/archive.zip?ref=master',
-            'Linux64': 'https://code.aliyun.com/vivid8710/aos-toolchain-linux64/repository/archive.zip?ref=master',
-            'Linux32': 'https://code.aliyun.com/vivid8710/aos-toolchain-linux64/repository/archive.zip?ref=master',
-            'OSX': 'https://code.aliyun.com/vivid8710/aos-toolchain-osx/repository/archive.zip?ref=master'
+            'Win32': 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-toolchain-windows.zip',
+            'Linux64': 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-toolchain-linux64.zip',
+            'Linux32': 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-toolchain-linux32.zip',
+            'OSX': 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-toolchain-osx.zip'
             }
         downloads['toolchain.zip'] = toolchains[host_os]
     if os.path.isdir('./build/OpenOCD') == False:
-        downloads['openocd.zip'] = 'https://code.aliyun.com/vivid8710/aos-openocd/repository/archive.zip?ref=master'
+        downloads['openocd.zip'] = 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-openocd.zip'
 
-    if downloads == {}:
-        return 'success'
 
+    if len(packages) > 0:
+        print("installing dependent packages...")
+        if sudo != '': print("please input your sudo password if asked")
+        print('installing: {0}...'.format(packages), end='')
+        for cmd in pre_install_cmd:
+            popen(cmd, shell=True)
+        sys.stdout.flush()
+        popen(install_cmd + ' '.join(packages), shell=True)
+        print('done')
+
+    remove_tmp_folder = False
     if os.path.isdir('./tmp') == False:
         os.mkdir('./tmp')
+        remove_tmp_folder = True
     os.chdir('./tmp')
     if host_os == 'Win32':
-        downloads['unzip.zip'] = 'https://code.aliyun.com/vivid8710/windows-unzip/repository/archive.zip?ref=master'
+        downloads['unzip.zip'] = 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/windows-unzip.zip'
+
+    if downloads == {}:
+        if packages == []: print("everything needed is already installed")
+        return 'success'
 
     for name in downloads:
-        try:
-            print "downloading {0} ...".format(name),
-            sys.stdout.flush()
-            urllib.urlretrieve(downloads[name], name)
-            print "succeed"
-        except:
-            print "failed"
-            os.chdir('../')
+        if download_url(downloads[name], name, retry=3) == False:
+            print("error: download {0} failed".format(name))
             return "fail"
 
-    print 'unzip files ...',
+    print('unzip files ...', end='')
     sys.stdout.flush()
     if host_os == 'Win32':
         zfobj = zipfile.ZipFile('unzip.zip')
         zfobj.extractall('./')
         zfobj.close()
-        unzip_dir = glob.glob('./windows-unzip-*')[0]
+        unzip_dir = 'windows-unzip'
         shutil.move(unzip_dir, 'win-unzip')
         unzip_program = os.getcwd() + '\\win-unzip\\7za.exe'
         unzip_pre_opt = ' x '
@@ -167,64 +206,77 @@ def __run_setup():
         unzip_cmd = unzip_program + unzip_pre_opt + filename + unzip_post_opt
         ret = os.system(unzip_cmd)
         if ret != 0:
-            print 'error: unzip {0} failed'.format(filename)
             os.chdir('..')
-            print 'failed'
+            print('failed')
+            error('unzip {0} failed'.format(filename))
             return
-        name = glob.glob('aos-toolchain-*.git')[0]
+        name = glob.glob('aos-toolchain-*')[0]
         shutil.move(name + '/compiler', '../build/compiler')
+        shutil.rmtree(name)
 
     filename = 'openocd.zip'
     if filename in downloads:
         unzip_cmd = unzip_program + unzip_pre_opt + filename + unzip_post_opt
         ret = os.system(unzip_cmd)
         if ret != 0:
-            print 'error: unzip {0} failed'.format(filename)
             os.chdir('..')
-            print 'failed'
+            print('failed')
+            error('unzip {0} failed'.format(filename))
             return
-        name = glob.glob('aos-openocd-*.git')[0]
+        name = 'aos-openocd'
         shutil.move(name + '/OpenOCD', '../build/OpenOCD')
-    print 'done'
+        shutil.rmtree(name)
+    print('done')
 
-    print 'cleanup...',
+    print('cleanup...', end='')
     sys.stdout.flush()
+    if 'unzip.zip' in downloads:
+        shutil.rmtree('win-unzip')
+    for filename in downloads:
+        os.remove(filename)
     os.chdir('../')
-    shutil.rmtree('./tmp', True)
-    print 'done'
-    print 'succeed'
+    if remove_tmp_folder: shutil.rmtree('./tmp', True)
+    print('done')
+    print('succeed')
 
 def install_esptool():
     if os.path.exists('./platform/mcu/esp32/esptool_py'):
         return
     try:
-        print "installing esptool ...",
+        print("installing esptool ...", end='')
         sys.stdout.flush()
         if os.path.isdir('./tmp') == False:
             os.mkdir('./tmp')
         os.chdir('./tmp')
-        urllib.urlretrieve('https://code.aliyun.com/vivid8710/aos-esptool/repository/archive.zip?ref=master', 'esptool.zip')
-        zfobj = zipfile.ZipFile('esptool.zip')
-        zfobj.extractall()
-        zfobj.close()
-        name = glob.glob('aos-esptool-*.git')[0]
+        esptool_url = 'http://alios-things-public.oss-cn-hangzhou.aliyuncs.com/aos-esptool.zip'
+        if download_url(esptool_url, 'esptool.zip', retry=3, printlog=False) == False:
+            print("failed, can not download esptool.zip")
+            return
+        if which('unzip') != None:
+            popen('unzip -qo esptool.zip', shell=True)
+        else:
+            zfobj = zipfile.ZipFile('esptool.zip')
+            zfobj.extractall()
+            zfobj.close()
+        name = 'aos-esptool'
         shutil.move(name + '/esptool_py', '../platform/mcu/esp32/esptool_py')
         shutil.rmtree(name)
         os.remove('esptool.zip')
         os.chdir('../')
-        print "succeed"
+        print("succeed")
     except:
         traceback.print_exc()
         os.chdir('../')
-        print "failed"
+        print("failed")
 
 def _run_make(arg_list):
     # check operating system
     host_os = get_host_os()
     if host_os == None:
-        error('Unsupported system!')
+        error('Unsupported Operating System!')
 
     #cd to aos root_dir
+    original_dir = os.getcwd()
     ret = cd_aos_root()
     if ret != 'success':
         return
@@ -245,6 +297,7 @@ def _run_make(arg_list):
     # Run make command
     make_cmd_str = ' '.join([make_cmd, 'HOST_OS=' + host_os, 'TOOLS_ROOT=' + tools_dir] + list(arg_list))
     popen(make_cmd_str, shell=True, cwd=os.getcwd())
+    if os.path.isdir(original_dir): os.chdir(original_dir)
 
 def is_path_component(path):
     if not os.path.exists(path):
@@ -284,7 +337,11 @@ def restore_aos_sdk(aos_sdk_path):
     with cd(aos_sdk_path):
         with open(os.devnull, 'w') as DEVNULL:
             try:
-                subprocess.Popen(['nohup', git_cmd, 'checkout', '-f'], stdout=DEVNULL, stderr=DEVNULL)
+                if sys.platform.startswith('win'):
+                    subprocess.Popen([git_cmd, 'checkout', '-f'], stdout=DEVNULL, stderr=DEVNULL)
+                else:
+                    subprocess.Popen(['nohup', git_cmd, 'checkout', '-f'], stdout=DEVNULL, stderr=DEVNULL)
+
             except ProcessException:
                 pass
 
@@ -307,6 +364,8 @@ def generate_aos_program(program_path, aos_sdk_path):
 
     copy_directory(os.path.join(aos_sdk_path, '.git'), os.path.join(program_aos_path, '.git'))
 
+    copy_directory(os.path.join(os.path.dirname(os.path.abspath(__file__)), '.vscode'), os.path.join(program_path, '.vscode'))
+
     restore_aos_sdk(aos_sdk_path)
 
     os.mkdir(os.path.join(program_path, 'remote'))
@@ -425,7 +484,6 @@ def new(name, scm='git', component=False, protocol=None):
         aos_sdk_path = Global().get_cfg(AOS_SDK_PATH)
         if aos_sdk_path and os.path.isdir(os.path.join(aos_sdk_path, '.git')):
             action('AliOS-Things code has been download \"%s\"' % aos_sdk_path)
-            restore_aos_sdk(aos_sdk_path)
         else:
             try:
                 change_path = Global().get_path()
@@ -504,7 +562,8 @@ def print_component_status(format):
                     cube_remove = False
 
                     rel_aos_path = os.path.dirname(relpath(aos_path, path))
-                    rel_path = os.path.join(os_name, rel_aos_path)
+                    rel_path = os.path.join(os_name, rel_aos_path).replace(os.path.sep, '/')
+
                     if cube_add_components and rel_aos_path in cube_add_components:
                         cube_add = True
                     if cube_remove_components and rel_aos_path in cube_remove_components:
@@ -1222,6 +1281,34 @@ def make():
         _run_make(['-e', '-f build/Makefile', make_args])
 
 
+# scons make command
+@subcommand('scons',
+            help='Make aos program/component by scons',
+            description="Make aos program/component by scons.")
+def scons():
+    # Get the make arguments
+    args = sys.argv[2:]
+    make_args = ['scons -f ucube.py']
+    for arg in args:
+        if '@' in arg:
+            targets = arg.split('@')
+            if len(targets) < 2:
+                error('Must special app and board when build aos')
+
+            app = 'APPLICATION='+targets[0]
+            board = 'BOARD=' + targets[1]
+            make_args.append(app)
+            make_args.append(board)
+
+            if len(targets) == 3:
+                build_type = 'TYPE=' + targets[3]
+                make_args.append(build_type)
+        else:
+            make_args.append(arg)
+
+    popen(' '.join(make_args), shell=True, cwd=os.getcwd())
+
+
 # Make command
 @subcommand('makelib',
             dict(name='path', nargs='?', help='Library directory path.'),
